#ifndef GAME_H
#define GAME_H

#include "client/input/Control.h"
#include "client/utils/Clock.h"
#include "client/rendering/RenderSettings.h"
#include "client/rendering/Renderer.h"
#include "client/rendering/Mesh.h"
#include "client/rendering/FileTexture.h"
#include "client/rendering/FontRenderer.h"
#include "common/world/World.h"
#include "common/block/BlockRegistry.h"
#include "rendering/renderer/WorldRenderer.h"

class Game final {
public:
    Game(const Control& control, const Clock& fps, const Clock& tps, RenderSettings& renderSettings);

    void tick();
    void renderWorld(float lag, Renderer& renderer) const;
    void renderTextOverlay(float lag, Renderer& renderer, FontRenderer& fr) const;

    bool isRunning() const;

    Vector splineTangent(const Vector& prev, const Vector& current, const Vector& next) const;
    Vector interpolate(const Vector& a, const Vector& b, const Vector& tanA, const Vector& tanB, float t) const;
    float distance(uint index, uint splits) const;
    void getPointsAndTangents(uint index, Vector& a, Vector& b, Vector& tanA, Vector& tanB) const;

private:
    const Control& control;
    const Clock& fps;
    const Clock& tps;
    RenderSettings& renderSettings;

    Vector lastPos;
    mutable Vector pos;
    Quaternion lastRotation;
    Quaternion rotation;

    BlockRegistry blockRegistry;
    World world;

    WorldRenderer worldRenderer;

    struct Point {
        Vector pos;
        float distance;
    };

    List<Point, 25> cameraPoints;
    uint pointIndex;
    float moveSpeed;
    float movedLength;

    enum Mode {
        AUTO, PLAYER
    };
    
    Mode mode;
};

#endif